* ---------------------------------------------------------------------------- *
* RDDMMSE.ADO: COMPUTE THE SHARP REGRESSION DISCONTINUITY ESTIMATOR
* ---------------------------------------------------------------------------- *
*
* DESCRIPTION:
*   - Estimate the sharp regression discontinuity estimator with the
*     bandwidths proposed by Arai and Ichimura (2017).
*   - Compute the robust and undersmoothing versions of the confidence
*     intervals.
*
* REFERENCE:
*   Arai and Ichimura (2017). "Simultaneous Selection of Optimal Bandwidths 
*   for the Sharp Regression Discontinuity Estimator," Quantitative
*   Economics, forthcoming.
*
* USAGE: 
*   rddmmse y x[, Cutoff(c 0)]
*
* INPUT ARGUMENTS:
*   y: Outcome variable 
*   x: Forcing variable 
*   c: Scalar cutoff (default = 0 is used when not specified)
*
* NOTES:
*   - Observations with either y or x is missing are not used for estimation.
*   - Stata version 13.0 or higher is required.
*
* OUTPUT & RETURN VALUES (available by `return list' after running the command):
*   tau_opt  - Treatment effect estimate
*   h_opt_m  - Optimal bandwidth left of the cutoff
*   h_opt_p  - Optimal bandwidth right of the cutoff
*   h_opt    - 1x2 vector of optimal bandwidth (h_opt_m, h_opt_p)
*   N_opt_m  - Number of observations within optimal bandwidth left of the cutoff
*   N_opt_p  - Number of observations within optimal bandwidth right of the cutoff
*   N_opt    - N_opt_m + N_opt_p
*   h_us_m   - Undersmoothing bandwidth left of the cutoff
*   h_us_p   - Undersmoothing bandwidth right of the cutoff
*   h_us     - 1x2 vector of undersmoothed bandwidth (h_us_m, h_us_p)
*   N_us_m   - Number of observations within undersmoothing bandwidth left of the cutoff
*   N_us_p   - Number of observations within undersmoothing bandwidth right of the cutoff
*   N_us     - N_us_m + N_us_p
*   se_c     - Conventional standard errors
*   se_r     - Robust standard errors
*   ci_nv_m  - Lower bound of non-valid confidence interval at 95%
*   ci_nv_p  - Upper bound of non-valid confidence interval at 95%
*   ci_nv    - 1x2 vector (ci_nv_m, ci_nv_p)
*   ci_c_m   - Lower bound of conventional confidence interval at 95%
*   ci_c_p   - Upper bound of conventional confidence interval at 95%
*   ci_c     - 1x2 vector (ci_c_m, ci_c_p)
*   ci_r_m   - Lower bound of robust confidence interval at 95%
*   ci_r_p   - Upper bound of robust confidence interval at 95%
*   ci_r     - 1x2 vector (ci_r_m, ci_r_p)
*   ci_us_m  - Lower bound of undersmoothing confidence interval at 95%
*   ci_us_p  - Upper bound of undersmoothing confidence interval at 95%
*   ci_us    - 1x2 vector (ci_us_m, ci_us_p)
*   bias_opt - Bias estimate with the optimal bandwidths
* 
* AUTHORS:
*   This code was written by Yoko Sakai & Masaru Nagashima.
*   Email (Sakai):     sakaiiyoko@gmail.com
*   Email (Nagashima): mnagashecon@gmail.com
*   We appreciate any feedbacks.
*
* DEVELOPMENT: 
*    07 May 2014: Original version of rddmmse.ado written.
*    20 Sep 2015: Minor changes in Step 2 and 3 (See Arai and Ichimura (2017))
*    23 Oct 2017: Confidence interval estimation added.
*    22 Nov 2017: Whole programme re-written with MATA.

cap pr drop rddmmse
pr def rddmmse, rclass
version 13.0

syntax varlist(min=2 max=2) [if] [in] [, Cutoff(real 0)]

preserve 
marksample touse

tokenize `varlist'
tempvar y x
g `y' = `1'
g `x' = `2' - `cutoff'
qui keep if `x' != . & `y' != . & `touse'

********************************************************************************
*** Compute treatment effect estimate                                        ***
********************************************************************************

mata: mata clear
mata: srdd("`y'", "`x'")

********************************************************************************
*** Compute confidence intervals                                             ***
********************************************************************************

tempname hopt_mat tau_val h_p_val h_m_val b2_p_val b2_m_val
matrix `hopt_mat' = (h_opt_m, h_opt_p)
loc `tau_val'     = tau
loc `h_p_val'     = h_opt_p
loc `h_m_val'     = h_opt_m
loc `b2_p_val'    = b2_p
loc `b2_m_val'    = b2_m

mata: mata clear
mata: srdd_ci("`y'", "`x'", ``tau_val'', ``h_p_val'', ``h_m_val'', ``b2_p_val'', ``b2_m_val'')

********************************************************************************
*** Display results                                                          ***
********************************************************************************

di ""
di "Estimation results: "
di "Treatment effect                        " %4.3f ``tau_val''
di "Bandwidths   [-,+]                      [" %4.3f ``h_m_val'' ", " %4.3f ``h_p_val'' "]"
di "Observations [-,+]                      ["         Nopt_m    ", "         Nopt_p    "]"
di "Bandwidths   [-,+] (undersmoothed)      [" %4.3f hus_m       ", " %4.3f  hus_p      "]"
di "Observations [-,+] (undersmoothed)      ["       Nus_m       ", "        Nus_p      "]"
di "Standard errors (conventional)          " %4.3f se_c
di "Standard errors (robust)                " %4.3f se_r
di "Robust t value                          " %4.3f t_r
di "P value                                 " %4.3f p_r
di "95% Confidence interval (non-valid)     [" %4.3f ci_nv_m ", " %4.3f ci_nv_p "]"
di "95% Confidence interval (conventional)  [" %4.3f ci_c_m  ", " %4.3f ci_c_p  "]"
di "95% Confidence interval (robust)        [" %4.3f ci_r_m  ", " %4.3f ci_r_p  "]"
di "95% Confidence interval (undersmoothed) [" %4.3f ci_us_m ", " %4.3f ci_us_p "]"

tempname N_opt N_us h_us ci_nv ci_c ci_r ci_us
scalar `N_opt' = Nopt_m + Nopt_p
scalar `N_us'  = Nus_m + Nus_p
mat `h_us'     = (hus_m, hus_p)
mat `ci_nv'    = (ci_nv_m, ci_nv_p)
mat `ci_c'     = (ci_c_m,  ci_c_p)
mat `ci_r'     = (ci_r_m,  ci_r_p)
mat `ci_us'    = (ci_us_m, ci_us_p)

return scalar bias_opt = bias_opt
return matrix ci_us    = `ci_us'
return scalar ci_us_p  = ci_us_p
return scalar ci_us_m  = ci_us_m
return matrix ci_r     = `ci_r'
return scalar ci_r_p   = ci_r_p
return scalar ci_r_m   = ci_r_m
return matrix ci_c     = `ci_c'
return scalar ci_c_p   = ci_c_p
return scalar ci_c_m   = ci_c_m
return matrix ci_nv    = `ci_nv'
return scalar ci_nv_p  = ci_nv_p
return scalar ci_nv_m  = ci_nv_m
return scalar p_r      = p_r
return scalar t_r      = t_r
return scalar se_r     = se_r
return scalar se_c     = se_c
return scalar N_us     = `N_us'
return scalar N_us_p   = Nus_p
return scalar N_us_m   = Nus_m
return matrix h_us     = `h_us'
return scalar h_us_p   = hus_p
return scalar h_us_m   = hus_m
return scalar N_opt    = `N_opt'
return scalar N_opt_p  = Nopt_p
return scalar N_opt_m  = Nopt_m
return matrix h_opt    = `hopt_mat'
return scalar h_opt_p  = ``h_p_val''
return scalar h_opt_m  = ``h_m_val''
return scalar tau_opt  = ``tau_val''

restore 

end
********************************************************************************
*** End of the main program                                                  ***
********************************************************************************

mata
void srdd(yy, xx) {
	y = st_data(., yy)
	x = st_data(., xx)
	
	// Step 1: Estimation of density and its first derivative ==================
	
	n        = rows(x)
	lambda   = sqrt(variance(x))
	a_pilot  = 2.34 * lambda / n^(1/5)
	f_k      = f_k(x, a_pilot)
	h1_pilot = lambda*(112*sqrt(pi())/n)^(1/7)
	f1_pilot = f1_pilot(x, h1_pilot)
	
	// Step 2: Get pilot bandwidths to estimate derivatives ====================
	
	// 4th order Polynomial regression -----------------------------------------
	
	Xbin_lt_x = (x :< 0)
	Xbin_bt_x = (x :>= 0)
	x2 = x  :* x
	x3 = x2 :* x
	x4 = x3 :* x
	nones   = J(n, 1, 1)
	y_m     = select(y,     Xbin_lt_x)
	nones_m = select(nones, Xbin_lt_x)
	x_m     = select(x,     Xbin_lt_x)
	x2_m    = select(x2,    Xbin_lt_x)
	x3_m    = select(x3,    Xbin_lt_x)
	x4_m    = select(x4,    Xbin_lt_x)
	y_p     = select(y,     Xbin_bt_x)
	nones_p = select(nones, Xbin_bt_x)
	x_p     = select(x,     Xbin_bt_x)
	x2_p    = select(x2,    Xbin_bt_x)
	x3_p    = select(x3,    Xbin_bt_x)
	x4_p    = select(x4,    Xbin_bt_x)
	data_mat_m = (y_m, nones_m, x_m, x2_m, x3_m, x4_m)
	data_mat_p = (y_p, nones_p, x_p, x2_p, x3_p, x4_p)
	num_X_lt_x = rows(x_m)
	num_X_bt_x = rows(x_p)
	
	YT_pc          = data_mat_p[.,1]
	TT_pc          = data_mat_p[.,(2..6)]
	lambda_pc_4_4g = invsym(TT_pc' * TT_pc)*(TT_pc' * YT_pc)
	m41_4          = 24 * lambda_pc_4_4g[5,1]
	
	YT_mc          = data_mat_m[.,1]
	TT_mc          = data_mat_m[.,(2..6)]
	lambda_mc_4_4g = invsym(TT_mc' * TT_mc)*(TT_mc' * YT_mc)
	m42_4          = 24 * lambda_mc_4_4g[5,1]
	
	vy_p_plugin_4g = sum((YT_pc-TT_pc*lambda_pc_4_4g):^2) / (num_X_bt_x - 5)
	vy_m_plugin_4g = sum((YT_mc-TT_mc*lambda_mc_4_4g):^2) / (num_X_lt_x - 5)
	
	C33K_T = Const_LPR(3,3)
	C23K_T = Const_LPR(2,3)
	h3_p   = C33K_T*(vy_p_plugin_4g/f_k/m41_4^2)^(1/9)/num_X_bt_x^(1/9)
	h3_m   = C33K_T*(vy_m_plugin_4g/f_k/m42_4^2)^(1/9)/num_X_lt_x^(1/9)
	h23_p  = C23K_T*(vy_p_plugin_4g/f_k/m41_4^2/num_X_bt_x)^(1/9)
	h23_m  = C23K_T*(vy_m_plugin_4g/f_k/m42_4^2/num_X_lt_x)^(1/9)
	
	// Step 3: Estimate the second and third derivatives =======================
	
	// 3rd order LPR for the second derivatives --------------------------------
	
	Xbin_bet_xxh_h3 = (abs(data_mat_p[.,3]) :<= h3_p)
	data_mat_h3_p_1 = select(data_mat_p[.,1], Xbin_bet_xxh_h3)
	data_mat_h3_p_2 = select(data_mat_p[.,2], Xbin_bet_xxh_h3)
	data_mat_h3_p_3 = select(data_mat_p[.,3], Xbin_bet_xxh_h3)
	data_mat_h3_p_4 = select(data_mat_p[.,4], Xbin_bet_xxh_h3)
	data_mat_h3_p_5 = select(data_mat_p[.,5], Xbin_bet_xxh_h3)
	YTh3_p = data_mat_h3_p_1
	Th3_p  = (data_mat_h3_p_2, data_mat_h3_p_3, data_mat_h3_p_4, data_mat_h3_p_5)
	Kvec_p = (data_mat_h3_p_2 - abs(data_mat_h3_p_3) / h3_p) :* (abs(data_mat_h3_p_3) :< h3_p)
	Wh3_p = diag(Kvec_p)
	lambda_p_all_33 = invsym(Th3_p'*Wh3_p*Th3_p) * (Th3_p'*Wh3_p*YTh3_p)
	
	Xbin_bet_xhx_h3 = (abs(data_mat_m[.,3]) :<= h3_m)
	data_mat_h3_m_1 = select(data_mat_m[.,1], Xbin_bet_xhx_h3)
	data_mat_h3_m_2 = select(data_mat_m[.,2], Xbin_bet_xhx_h3)
	data_mat_h3_m_3 = select(data_mat_m[.,3], Xbin_bet_xhx_h3)
	data_mat_h3_m_4 = select(data_mat_m[.,4], Xbin_bet_xhx_h3)
	data_mat_h3_m_5 = select(data_mat_m[.,5], Xbin_bet_xhx_h3)
	YTh3_m = data_mat_h3_m_1
	Th3_m  = (data_mat_h3_m_2, data_mat_h3_m_3, data_mat_h3_m_4, data_mat_h3_m_5)
	Kvec_m = (data_mat_h3_m_2 - abs(data_mat_h3_m_3) / h3_m) :* (abs(data_mat_h3_m_3) :< h3_m)
	Wh3_m  = diag(Kvec_m)
	lambda_m_all_33 = invsym(Th3_m'*Wh3_m*Th3_m) * (Th3_m'*Wh3_m*YTh3_m)
	
	m3c_p = 6*lambda_p_all_33[4,1]
	m3c_m = 6*lambda_m_all_33[4,1]
	
	// 3rd order LPR for the third derivatives ---------------------------------
	
	Xbin_bet_xxh_h23 = (abs(data_mat_p[.,3]) :<= h23_p)
	data_mat_h23_p_1 = select(data_mat_p[.,1], Xbin_bet_xxh_h23)
	data_mat_h23_p_2 = select(data_mat_p[.,2], Xbin_bet_xxh_h23)
	data_mat_h23_p_3 = select(data_mat_p[.,3], Xbin_bet_xxh_h23)
	data_mat_h23_p_4 = select(data_mat_p[.,4], Xbin_bet_xxh_h23)
	data_mat_h23_p_5 = select(data_mat_p[.,5], Xbin_bet_xxh_h23)
	YTh23_p   = data_mat_h23_p_1
	Th23_p    = (data_mat_h23_p_2, data_mat_h23_p_3, data_mat_h23_p_4, data_mat_h23_p_5)
	Kvech23_p = (data_mat_h23_p_2 - abs(data_mat_h23_p_3) / h23_p) :* (abs(data_mat_h23_p_3) :< h23_p)
	Wh23_p    = diag(Kvech23_p)
	lambda_p_all_23 = invsym(Th23_p'*Wh23_p*Th23_p) * (Th23_p'*Wh23_p*YTh23_p)
	m2c_p_2   = 2 * lambda_p_all_23[3,1]
	
	YTh23_p_hat = Th23_p*lambda_p_all_23
	vy_p_4_2 = (YTh23_p - YTh23_p_hat)'*Wh23_p*(YTh23_p - YTh23_p_hat) / trace(Wh23_p-Wh23_p*Th23_p*invsym(Th23_p'*Wh23_p*Th23_p)*Th23_p'*Wh23_p)
	
	Xbin_bet_xhx_h23 = (abs(data_mat_m[.,3]) :<= h23_m)
	data_mat_h23_m_1 = select(data_mat_m[.,1], Xbin_bet_xhx_h23)
	data_mat_h23_m_2 = select(data_mat_m[.,2], Xbin_bet_xhx_h23)
	data_mat_h23_m_3 = select(data_mat_m[.,3], Xbin_bet_xhx_h23)
	data_mat_h23_m_4 = select(data_mat_m[.,4], Xbin_bet_xhx_h23)
	data_mat_h23_m_5 = select(data_mat_m[.,5], Xbin_bet_xhx_h23)
	YTh23_m   = data_mat_h23_m_1
	Th23_m    = (data_mat_h23_m_2, data_mat_h23_m_3, data_mat_h23_m_4, data_mat_h23_m_5)
	Kvech23_m = (data_mat_h23_m_2 - abs(data_mat_h23_m_3) / h23_m) :* (abs(data_mat_h23_m_3) :< h23_m)
	Wh23_m    = diag(Kvech23_m)
	lambda_m_all_23 = invsym(Th23_m'*Wh23_m*Th23_m) * (Th23_m'*Wh23_m*YTh23_m)
	m2c_m_2   = 2 * lambda_m_all_23[3,1]
	
	YTh23_m_hat = Th23_m*lambda_m_all_23
	vy_m_4_2 = (YTh23_m - YTh23_m_hat)'*Wh23_m*(YTh23_m - YTh23_m_hat) / trace(Wh23_m-Wh23_m*Th23_m*invsym(Th23_m'*Wh23_m*Th23_m)*Th23_m'*Wh23_m)
	
	m2_pilot = (m2c_p_2, m2c_m_2)
	m3_pilot = (m3c_p,   m3c_m)
	v_pilot = (vy_p_4_2, vy_m_4_2)
	
	// Step 4: Get bandwidths that minimize MMSE ===============================
	
	NN_min_AI = 3
	X_lt_x = data_mat_m[.,3]
	X_bt_x = data_mat_p[.,3]
	sorted_X_bt_x         = sort(X_bt_x,1)       // ascending order on 1st arg.
	sorted_X_lt_x_des_abs = sort(abs(X_lt_x), 1) // 
	NN_p_AI = sorted_X_bt_x[NN_min_AI,1]
	NN_m_AI = sorted_X_lt_x_des_abs[NN_min_AI,1]
	h_lb = (NN_p_AI, NN_m_AI)
	
	hu1 = sorted_X_bt_x[num_X_bt_x,1]
	hu2 = sorted_X_lt_x_des_abs[num_X_lt_x,1]
	h_ub = (hu1, hu2)
	
	h1_range = hu1 - NN_p_AI
	h2_range = hu2 - NN_m_AI
	h_range = (h1_range, h2_range)
	
	external h_mat_local, min_val
	h_mat_local = J(100,2,0)
	min_val     = J(100,1,0)
	
	for(i = 1; i <= 10; i++) {
		for(j = 1; j <= 10; j++) {
			ret_vec  = do_optim(h_lb, h_range, i, j, f_k, f1_pilot, m2_pilot, m3_pilot, v_pilot, n)
			h_mat_local[10*(i-1)+j, 1] = ret_vec[1]
			h_mat_local[10*(i-1)+j, 2] = ret_vec[2]
			min_val[10*(i-1)+j, 1]     = ret_vec[3]
		}
	}
	min_index = J(1,1,.)
	minindex(min_val, 1, min_index, somevector)
	MMSE  = min_val[min_index]
	h_opt = h_mat_local[min_index, (1..2)]
	
	// Compute SRDD estimate ===================================================
	
	Xbin_bet_xxh_opt = Xbin_bt_x :* (x :<= h_opt[1])
	Xbin_bet_xhx_opt = Xbin_lt_x :* (x :>= -h_opt[2])
	n_p = sum(Xbin_bet_xxh_opt)
	n_m = sum(Xbin_bet_xhx_opt)
	
	data_trim_mat_p_opt_1 = select(y,     Xbin_bet_xxh_opt)
	data_trim_mat_p_opt_2 = select(nones, Xbin_bet_xxh_opt)
	data_trim_mat_p_opt_3 = select(x,     Xbin_bet_xxh_opt)
	data_trim_mat_p_opt = (data_trim_mat_p_opt_1, data_trim_mat_p_opt_2, data_trim_mat_p_opt_3)
	K_p_opt = (data_trim_mat_p_opt[.,2] - abs(data_trim_mat_p_opt[.,3]) / h_opt[1])
	W_p_opt = diag(K_p_opt)
	Y_p_opt = data_trim_mat_p_opt[.,1]
	T_p_opt = data_trim_mat_p_opt[.,(2..3)]
	lambda_p_all_opt = invsym(T_p_opt'*W_p_opt*T_p_opt) * (T_p_opt'*W_p_opt*Y_p_opt)
	delta_p_opt      = lambda_p_all_opt[1]
	
	data_trim_mat_m_opt_1 = select(y,     Xbin_bet_xhx_opt)
	data_trim_mat_m_opt_2 = select(nones, Xbin_bet_xhx_opt)
	data_trim_mat_m_opt_3 = select(x,     Xbin_bet_xhx_opt)
	data_trim_mat_m_opt = (data_trim_mat_m_opt_1, data_trim_mat_m_opt_2, data_trim_mat_m_opt_3)
	K_m_opt = (data_trim_mat_m_opt[.,2] - abs(data_trim_mat_m_opt[.,3]) / h_opt[2])
	W_m_opt = diag(K_m_opt)
	Y_m_opt = data_trim_mat_m_opt[.,1]
	T_m_opt = data_trim_mat_m_opt[.,(2..3)]
	lambda_m_all_opt = invsym(T_m_opt'*W_m_opt*T_m_opt) * (T_m_opt'*W_m_opt*Y_m_opt)
	delta_m_opt      = lambda_m_all_opt[1]
	
	tau = delta_p_opt - delta_m_opt
	
	st_numscalar("tau", tau)
	st_numscalar("h_opt_p", h_opt[1])
	st_numscalar("h_opt_m", h_opt[2])
	st_numscalar("b2_p", h23_p)
	st_numscalar("b2_m", h23_m)
	st_numscalar("b3_p", h3_p)
	st_numscalar("b3_m", h3_m)
}
end

mata
real vector do_optim(h_lb, h_range, i, j, f_k, f1_pilot, m2_pilot, m3_pilot, v_pilot, n) {
	external ext_f_pilot, ext_f1_pilot, ext_m2_pilot, ext_m3_pilot, ext_v_pilot, ext_n, h_mat_local, min_val
	
	ext_f_pilot  = f_k
	ext_f1_pilot = f1_pilot
	ext_m2_pilot = m2_pilot
	ext_m3_pilot = m3_pilot
	ext_v_pilot  = v_pilot
	ext_n        = n
	
	h_ini   = (h_lb[1]+0.1*i*h_range[1], h_lb[2]+0.1*j*h_range[2])
	
	S = optimize_init()
	
	optimize_init_evaluator(S, &objfun())
	optimize_init_which(S, "min")
	optimize_init_evaluatortype(S, "d0")
	optimize_init_params(S, h_ini)
	
//	optimize_init_technique(S,"nr")     // flat / discont.
//	optimize_init_technique(S,"dfp")    // slow
	optimize_init_technique(S,"bfgs")
//	optimize_init_technique(S,"nm")     // params below required
//	ptol = optimize_init_conv_ptol(S)
//	deltas = (ptol*100000, ptol*100000)
//	deltas = (1, 1)
//	deltas = 10*h_ini
//	deltas = h_ini
//	optimize_init_nmsimplexdeltas(S, deltas)
	
	optimize_init_conv_maxiter(S, 10000)
	optimize_init_trace_dots(S, "off")
	optimize_init_trace_value(S, "off")
	optimize_init_trace_tol(S, "off")
	optimize_init_trace_step(S, "off")
	optimize_init_trace_paramdiffs(S, "off")
	optimize_init_trace_params(S, "off")
	optimize_init_trace_gradient(S, "off")
	optimize_init_trace_Hessian(S, "off")
	optimize_init_conv_warning(S, "off")
	optimize_init_verbose(S, 0)
	optimize_init_tracelevel(S, "none")
	
	rc   = _optimize(S)
	h    = optimize_result_params(S)
	MMSE = optimize_result_value(S)
	if (optimize_result_converged(S) & MMSE >= 0) {
		ret_vec = (h[1], h[2], MMSE)
	}
	else {
		ret_vec = (., ., .)
	}
	return(ret_vec)
}
end

mata
void objfun(todo, h, Qn,  g, H) {
	external ext_f_pilot, ext_f1_pilot, ext_m2_pilot, ext_m3_pilot, ext_v_pilot, ext_n
	
	k_const    = (-0.1, -0.1, -0.08, 24/5)
	b2         = (0,0)
	m2foverf12 = (0,0)
	for(j=1; j<=2; j++) {
		m2foverf12[j] = ext_m2_pilot[j] * ext_f1_pilot / ext_f_pilot / 2
		b2[j] = (-1)^(j+1) * (k_const[2] * (m2foverf12[j] + ext_m3_pilot[j]/6) - k_const[3]*m2foverf12[j])
	}
	
	h2order = ext_m2_pilot[1]*h[1]^2 - ext_m2_pilot[2]*h[2]^2
	h3order = b2[1]*h[1]^3 - b2[2]*h[2]^3
	vterm   = ext_v_pilot[1]/h[1] + ext_v_pilot[2]/h[2]
	Qn = (k_const[1]^2)*(h2order^2)/4 + h3order^2 + k_const[4]*(vterm) / ext_f_pilot / ext_n
}
end

mata
real scalar Const_LPR(v, p) {
	S6_1 = (1/2,  1/6 , 1/12, 1/20,  1/30,  1/42,  1/56)
	S6_2 = (1/6,  1/12, 1/20, 1/30,  1/42,  1/56,  1/72)
	S6_3 = (1/12, 1/20, 1/30, 1/42,  1/56,  1/72,  1/90)
	S6_4 = (1/20, 1/30, 1/42, 1/56,  1/72,  1/90,  1/110)
	S6_5 = (1/30, 1/42, 1/56, 1/72,  1/90,  1/110, 1/132)
	S6_6 = (1/42, 1/56, 1/72, 1/90,  1/110, 1/132, 1/156)
	S6_7 = (1/56, 1/72, 1/90, 1/110, 1/132, 1/156, 1/182)
	S6   = (S6_1 \ S6_2 \ S6_3 \ S6_4 \ S6_5 \ S6_6 \ S6_7)
	K6_1 = (1/3,   1/12,  1/30,  1/60,  1/105, 1/168,  1/252 )
	K6_2 = (1/12,  1/30,  1/60,  1/105, 1/168, 1/252,  1/360 )
	K6_3 = (1/30,  1/60,  1/105, 1/168, 1/252, 1/360,  1/495 )
	K6_4 = (1/60,  1/105, 1/168, 1/252, 1/360, 1/495,  1/660 )
	K6_5 = (1/105, 1/168, 1/252, 1/360, 1/495, 1/660,  1/858 )
	K6_6 = (1/168, 1/252, 1/360, 1/495, 1/660, 1/858,  1/1092)
	K6_7 = (1/252, 1/360, 1/495, 1/660, 1/858, 1/1092, 1/1365)
	K6   = (K6_1 \ K6_2 \ K6_3 \ K6_4 \ K6_5 \ K6_6 \ K6_7)
	mu   = (1/6, 1/12, 1/20, 1/30, 1/42, 1/56, 1/72, 1/90, 1/110, 1/132, 1/156, 1/182, 1/210)
	Sinv_p = invsym(S6[(1..p+1),(1..p+1)])
	int_tp_K = Sinv_p[v+1,.] * mu[1,((p+1)..(2*p+1))]'
	int_K2 = Sinv_p[v+1,.] * K6[(1..(p+1)),(1..p+1)] * Sinv_p[v+1,.]'
	R = ((factorial(p+1)^2 * (2*v + 1) * int_K2) / (2 * (p+1-v) * int_tp_K^2))^(1/(2*p + 3))
	
	return(R)
}
end

mata 
void srdd_ci(yy, xx, tt, hhp, hhm, hhp23, hhm23) {
	// Prepare data ============================================================
	
	y = st_data(., yy)
	x = st_data(., xx)
	tau_c = tt
	h_mmse = (hhp, hhm)
	h_opt  = h_mmse
	b2 = h_mmse
	b3 = h_mmse
	
	n = rows(x)
	x  = x
	x2 = x  :* x
	x3 = x2 :* x
	x4 = x3 :* x
	nones = J(n, 1, 1)
	data_mat = (y, nones, x, x2, x3, x4)
	Xbin_lt_x = (x :< 0)
	Xbin_bt_x = (x :>= 0)
	y_m     = select(y,     Xbin_lt_x)
	nones_m = select(nones, Xbin_lt_x)
	x_m     = select(x,     Xbin_lt_x)
	x2_m    = select(x2,    Xbin_lt_x)
	x3_m    = select(x3,    Xbin_lt_x)
	x4_m    = select(x4,    Xbin_lt_x)
	data_mat_m = (y_m, nones_m, x_m, x2_m, x3_m, x4_m)
	y_p     = select(y,     Xbin_bt_x)
	nones_p = select(nones, Xbin_bt_x)
	x_p     = select(x,     Xbin_bt_x)
	x2_p    = select(x2,    Xbin_bt_x)
	x3_p    = select(x3,    Xbin_bt_x)
	x4_p    = select(x4,    Xbin_bt_x)
	data_mat_p = (y_p, nones_p, x_p, x2_p, x3_p, x4_p)
	
	// Check the sign of the second derivatives ================================
	
	h23_p = hhp23
	h23_m = hhm23
	Xbin_bet_xxh_h23 = (x :>= 0 :& x :<= h23_p)
	Xbin_bet_xhx_h23 = (x :<  0 :& x :>= (-1)*h23_m)
	
	y_h23_p     = select(y,     Xbin_bet_xxh_h23)
	nones_h23_p = select(nones, Xbin_bet_xxh_h23)
	x_h23_p     = select(x,     Xbin_bet_xxh_h23)
	x2_h23_p    = select(x2,    Xbin_bet_xxh_h23)
	x3_h23_p    = select(x3,    Xbin_bet_xxh_h23)
	x4_h23_p    = select(x4,    Xbin_bet_xxh_h23)
	Th23_p      = (nones_h23_p, x_h23_p, x2_h23_p, x3_h23_p)
	Kvech23_p   = (nones_h23_p - abs(x_h23_p)/h23_p)
	Wh23_p      = diag(Kvech23_p)
	lambda_p_all_23 = invsym(Th23_p' * Wh23_p * Th23_p) * (Th23_p' * Wh23_p * y_h23_p)
	m2c_p_2     = 2*lambda_p_all_23[3,1]
	
	y_h23_m     = select(y,     Xbin_bet_xhx_h23)
	nones_h23_m = select(nones, Xbin_bet_xhx_h23)
	x_h23_m     = select(x,     Xbin_bet_xhx_h23)
	x2_h23_m    = select(x2,    Xbin_bet_xhx_h23)
	x3_h23_m    = select(x3,    Xbin_bet_xhx_h23)
	x4_h23_m    = select(x4,    Xbin_bet_xhx_h23)
	Th23_m      = (nones_h23_m, x_h23_m, x2_h23_m, x3_h23_m)
	Kvech23_m   = (nones_h23_m - abs(x_h23_m)/h23_m)
	Wh23_m      = diag(Kvech23_m)
	lambda_m_all_23 = invsym(Th23_m' * Wh23_m * Th23_m) * (Th23_m' * Wh23_m * y_h23_m)
	m2c_m_2     = 2*lambda_m_all_23[3,1]
	
	if ((m2c_p_2 * m2c_m_2) >= 0) {
		h_opt = n^(-1/25) * h_opt
	}
	
	// Estimation of density and its first derivative ==========================
	
	lambda  = sqrt(variance(x))
	a_pilot = 2.34 * lambda / (n^(1/5))
	f_k     = f_k(x, a_pilot)
	h1_pilot = lambda*(112*sqrt(pi())/n)^(1/7)
	f1_pilot = f1_pilot(x, h1_pilot)
	
	// Compute tau and V1 ======================================================
	
	e0 = (1 \ 0)
	h_vec_p = (1, (1/h_opt[1,1]), (1/h_opt[1,1])^2, (1/h_opt[1,1])^3)
	h_vec_m = (1, (1/h_opt[1,2]), (1/h_opt[1,2])^2, (1/h_opt[1,2])^3)
	H_p2 = diag(h_vec_p[1,(1,2)])
	H_m2 = diag(h_vec_m[1,(1,2)])
	H_p3 = diag(h_vec_p)
	H_m3 = diag(h_vec_m)
	
	X3h_p = data_mat_p[.,(2..5)] * H_p3
	X3h_m = data_mat_m[.,(2..5)] * H_m3
	Kind_h_p     = (abs(data_mat_p[.,3]) :<  h_opt[1,1])
	Kind_h_m     = (abs(data_mat_m[.,3]) :<  h_opt[1,2])
	Kvec_h_p     = (data_mat_p[.,2] - abs(data_mat_p[.,3])/h_opt[1,1]) :* Kind_h_p
	Kvec_h_m     = (data_mat_m[.,2] - abs(data_mat_m[.,3])/h_opt[1,2]) :* Kind_h_m
	Kvec_h_p2    = select(Kvec_h_p, Kind_h_p)
	Kvec_h_m2    = select(Kvec_h_m, Kind_h_m)
	yh_p         = select(data_mat_p[.,1], Kind_h_p)
	yh_m         = select(data_mat_m[.,1], Kind_h_m)
	num_yh_p     = rows(yh_p)
	num_yh_m     = rows(yh_m)
	num          = (num_yh_m, num_yh_p)
	X3h_p2_1     = select(X3h_p[.,1], Kind_h_p)
	X3h_p2_2     = select(X3h_p[.,2], Kind_h_p)
	X3h_p2       = (X3h_p2_1, X3h_p2_2)
	X3h_m2_1     = select(X3h_m[.,1], Kind_h_m)
	X3h_m2_2     = select(X3h_m[.,2], Kind_h_m)
	X3h_m2       = (X3h_m2_1, X3h_m2_2)
	Wh_p         = diag(Kvec_h_p2)
	Wh_m         = diag(Kvec_h_m2)
	Gamma_h_p    = X3h_p2' * Wh_p * X3h_p2 / n
	Gamma_h_m    = X3h_m2' * Wh_m * X3h_m2 / n
	XWY_h_p    = X3h_p2' * Wh_p * yh_p / n
	XWY_h_m    = X3h_m2' * Wh_m * yh_m / n
	invGamma_h_p = invsym(Gamma_h_p)
	invGamma_h_m = invsym(Gamma_h_m)
	delta_p_opt  = e0' * invGamma_h_p * XWY_h_p
	delta_m_opt  = e0' * invGamma_h_m * XWY_h_m
	tau          = delta_p_opt - delta_m_opt
	beta_p       = invGamma_h_p * XWY_h_p
	beta_m       = invGamma_h_m * XWY_h_m
	YT_p_hat     = X3h_p2 * beta_p
	YT_m_hat     = X3h_m2 * beta_m
	s2_p         = (yh_p - YT_p_hat)'*Wh_p*(yh_p - YT_p_hat)/trace(Wh_p - Wh_p*X3h_p2*invsym(X3h_p2'*Wh_p*X3h_p2)*X3h_p2'*Wh_p)
	s2_m         = (yh_m - YT_m_hat)'*Wh_m*(yh_m - YT_m_hat)/trace(Wh_m - Wh_m*X3h_m2*invsym(X3h_m2'*Wh_m*X3h_m2)*X3h_m2'*Wh_m)
	
	Psi_h_p = s2_p*X3h_p2'*Wh_p*Wh_p*X3h_p2/n
	Psi_h_m = s2_m*X3h_m2'*Wh_m*Wh_m*X3h_m2/n
	Vh_p    = (e0'*invGamma_h_p*Psi_h_p*invGamma_h_p*e0)/n
	Vh_m    = (e0'*invGamma_h_m*Psi_h_m*invGamma_h_m*e0)/n
	V1      = Vh_p + Vh_m
	se_c    = sqrt(V1)
	
	// Compute V2 ==============================================================
	
	e2 = (0 \ 0 \ 1 \ 0)
	b2_vec_p   = (1, (1/b2[1,1]), (1/b2[1,1])^2, (1/b2[1,1])^3)
	b2_vec_m   = (1, (1/b2[1,2]), (1/b2[1,2])^2, (1/b2[1,2])^3)
	Hb2_p      = diag(b2_vec_p)
	Hb2_m      = diag(b2_vec_m)
	X3b2_p     = data_mat_p[.,(2..5)] * Hb2_p
	X3b2_m     = data_mat_m[.,(2..5)] * Hb2_m
	Kind_b2_p  = (abs(data_mat_p[.,3]) :< b2[1,1])
	Kind_b2_m  = (abs(data_mat_m[.,3]) :< b2[1,2])
	Kvec_b2_p  = (data_mat_p[.,2] - abs(data_mat_p[.,3]) / b2[1,1]) :* Kind_b2_p
	Kvec_b2_m  = (data_mat_m[.,2] - abs(data_mat_m[.,3]) / b2[1,2]) :* Kind_b2_m
	yb2_p      = select(data_mat_p[.,1], Kind_b2_p)
	yb2_m      = select(data_mat_m[.,1], Kind_b2_m)
	X3b2_p2_1  = select(X3b2_p[.,1], Kind_b2_p)
	X3b2_p2_2  = select(X3b2_p[.,2], Kind_b2_p)
	X3b2_p2_3  = select(X3b2_p[.,3], Kind_b2_p)
	X3b2_p2_4  = select(X3b2_p[.,4], Kind_b2_p)
	X3b2_p2    = (X3b2_p2_1, X3b2_p2_2, X3b2_p2_3, X3b2_p2_4)
	X3b2_m2_1  = select(X3b2_m[.,1], Kind_b2_m)
	X3b2_m2_2  = select(X3b2_m[.,2], Kind_b2_m)
	X3b2_m2_3  = select(X3b2_m[.,3], Kind_b2_m)
	X3b2_m2_4  = select(X3b2_m[.,4], Kind_b2_m)
	X3b2_m2    = (X3b2_m2_1, X3b2_m2_2, X3b2_m2_3, X3b2_m2_4)
	Kvec_b2_p2 = select(Kvec_b2_p, Kind_b2_p)
	Kvec_b2_m2 = select(Kvec_b2_m, Kind_b2_m)
	Wb2_p      = diag(Kvec_b2_p2)
	Wb2_m      = diag(Kvec_b2_m2)
	Gamma_b2_p = X3b2_p2' * Wb2_p * X3b2_p2 / n
	Gamma_b2_m = X3b2_m2' * Wb2_m * X3b2_m2 / n
	invGamma_b2_p = invsym(Gamma_b2_p)
	invGamma_b2_m = invsym(Gamma_b2_m)
	XWY_b2_p   = X3b2_p2' * Wb2_p * yb2_p / n
	XWY_b2_m   = X3b2_m2' * Wb2_m * yb2_m / n
	m2_p       = 2 * e2' * Hb2_p * invGamma_b2_p * XWY_b2_p
	m2_m       = 2 * e2' * Hb2_m * invGamma_b2_m * XWY_b2_m
	
	Psi_b2_p   = s2_p * X3b2_p2' * Wb2_p * Wb2_p * X3b2_p2 / n
	Psi_b2_m   = s2_m * X3b2_m2' * Wb2_m * Wb2_m * X3b2_m2 / n
	V23_b_p    = b2[1,1]^(-4) * (e2' * invGamma_b2_p * Psi_b2_p * invGamma_b2_p * e2) / n
	V23_b_m    = b2[1,2]^(-4) * (e2' * invGamma_b2_m * Psi_b2_m * invGamma_b2_m * e2) / n
	X3_xx_h_p  = select(X3h_p[.,3], Kind_h_p)
	X3_xx_h_m  = select(X3h_m[.,3], Kind_h_m)
	Theta2_h_p = X3h_p2' * Wh_p * X3_xx_h_p / n
	Theta2_h_m = X3h_m2' * Wh_m * X3_xx_h_m / n
	B012_p     = e0' * H_p2 * invGamma_h_p * Theta2_h_p
	B012_m     = e0' * H_m2 * invGamma_h_m * Theta2_h_m
	V2_p       = h_opt[1,1]^4 * V23_b_p * (-0.1)^2
	V2_m       = h_opt[1,2]^4 * V23_b_m * (-0.1)^2
	V2         = V2_p + V2_m
	
	// Compute V3_1 ============================================================
	
	B31_const = (-0.02) * f1_pilot / f_k
	V31_p     = h_opt[1,1]^6 * V23_b_p * B31_const^2
	V31_m     = h_opt[1,2]^6 * V23_b_m * B31_const^2
	V31       = V31_p + V31_m
	
	// Compute V3_2 ============================================================
	
	e3         = (0 \ 0 \ 0 \ 1)
	b3_vec_p   = (1, (1/b3[1,1]), (1/b3[1,1])^2, (1/b3[1,1])^3)
	b3_vec_m   = (1, (1/b3[1,2]), (1/b3[1,2])^2, (1/b3[1,2])^3)
	Hb3_p      = diag(b3_vec_p)
	Hb3_m      = diag(b3_vec_m)
	X3b3_p     = data_mat_p[.,(2..5)] * Hb3_p
	X3b3_m     = data_mat_m[.,(2..5)] * Hb3_m
	Kind_b3_p  = (abs(data_mat_p[.,3]) :< b3[1,1])
	Kind_b3_m  = (abs(data_mat_m[.,3]) :< b3[1,2])
	Kvec_b3_p  = (data_mat_p[.,2] - abs(data_mat_p[.,3]) / b3[1,1]) :* Kind_b3_p
	Kvec_b3_m  = (data_mat_m[.,2] - abs(data_mat_m[.,3]) / b3[1,2]) :* Kind_b3_m
	yb3_p      = select(data_mat_p[.,1], Kind_b3_p)
	yb3_m      = select(data_mat_m[.,1], Kind_b3_m)
	X3b3_p2_1  = select(X3b3_p[.,1], Kind_b3_p)
	X3b3_p2_2  = select(X3b3_p[.,2], Kind_b3_p)
	X3b3_p2_3  = select(X3b3_p[.,3], Kind_b3_p)
	X3b3_p2_4  = select(X3b3_p[.,4], Kind_b3_p)
	X3b3_p2    = (X3b3_p2_1, X3b3_p2_2, X3b3_p2_3, X3b3_p2_4)
	X3b3_m2_1  = select(X3b3_m[.,1], Kind_b3_m)
	X3b3_m2_2  = select(X3b3_m[.,2], Kind_b3_m)
	X3b3_m2_3  = select(X3b3_m[.,3], Kind_b3_m)
	X3b3_m2_4  = select(X3b3_m[.,4], Kind_b3_m)
	X3b3_m2    = (X3b3_m2_1, X3b3_m2_2, X3b3_m2_3, X3b3_m2_4)
	Kvec_b3_p2 = select(Kvec_b3_p, Kind_b3_p)
	Kvec_b3_m2 = select(Kvec_b3_m, Kind_b3_m)
	Wb3_p      = diag(Kvec_b3_p2)
	Wb3_m      = diag(Kvec_b3_m2)
	invGamma_b3_p = invsym(X3b3_p2' * Wb3_p * X3b3_p2 / n)
	invGamma_b3_m = invsym(X3b3_m2' * Wb3_m * X3b3_m2 / n)
	XWY_b3_p   = X3b3_p2' * Wb3_p * yb3_p / n
	XWY_b3_m   = X3b3_m2' * Wb3_m * yb3_m / n
	m3_p       = 6 * e3' * Hb3_p * invGamma_b3_p * XWY_b3_p
	m3_m       = 6 * e3' * Hb3_m * invGamma_b3_m * XWY_b3_m
	
	Psi_b3_p   = s2_p * X3b3_p2' * Wb3_p * Wb3_p * X3b3_p2 / n
	Psi_b3_m   = s2_m * X3b3_m2' * Wb3_m * Wb3_m * X3b3_m2 / n
	V33_b_p    = b3[1,1]^(-6) * (e3' * invGamma_b3_p * Psi_b3_p * invGamma_b3_p * e3) / n
	V33_b_m    = b3[1,2]^(-6) * (e3' * invGamma_b3_m * Psi_b3_m * invGamma_b3_m * e3) / n
	X3_xxx_h_p = select(X3h_p[.,4], Kind_h_p)
	X3_xxx_h_m = select(X3h_m[.,4], Kind_h_m)
	Theta3_h_p = X3h_p2' * Wh_p * X3_xxx_h_p / n
	Theta3_h_m = X3h_m2' * Wh_m * X3_xxx_h_m / n
	B013_p     = e0' * H_p2 * invGamma_h_p * Theta3_h_p
	B013_m     = e0' * H_m2 * invGamma_h_m * Theta3_h_m
	
	V32_p      = h_opt[1,1]^6 * V33_b_p * (-0.1)^2
	V32_m      = h_opt[1,2]^6 * V33_b_m * (-0.1)^2
	V32        = V32_p + V32_m
	
	if(h_opt[1,1] <= b2[1,1]){
		Kvec_b2_p3  = select(Kvec_b2_p, Kind_h_p)
		Wb2_p2      = diag(Kvec_b2_p3)
		X3b2_p3     = select(X3b2_p, Kind_h_p)
		Psi13_hb2_p = s2_p * X3h_p2' * Wh_p * Wb2_p2 * X3b2_p3 / n
	}
	else {
		Kvec_h_p2   = select(Kvec_h_p, Kind_b2_p)
		Wh_p2       = diag(Kvec_h_p2)
		X3h_p3      = select(X3h_p[.,(1,2)], Kind_b2_p)
		Psi13_hb2_p = s2_p * X3h_p3' * Wh_p2 * Wb2_p * X3b2_p2 / n
	}
	if(h_opt[1,2] <= b2[1,2]){
		Kvec_b2_m3  = select(Kvec_b2_m, Kind_h_m)
		Wb2_m2      = diag(Kvec_b2_m3)
		X3b2_m3     = select(X3b2_m, Kind_h_m)
		Psi13_hb2_m = s2_m * X3h_m2' * Wh_m * Wb2_m2 * X3b2_m3 / n
	}
	else {
		Kvec_h_m2   = select(Kvec_h_m, Kind_b2_m)
		Wh_m2       = diag(Kvec_h_m2);
		X3h_m3      = select(X3h_m[.,(1,2)], Kind_b2_m)
		Psi13_hb2_m = s2_m * X3h_m3' * Wh_m2 * Wb2_m * X3b2_m2 / n
	}
	
	D12_hb2_p = b2[1,1]^(-2) * e0' * invGamma_h_p * Psi13_hb2_p * invGamma_b2_p * e2 / n
	D12_hb2_m = b2[1,2]^(-2) * e0' * invGamma_h_m * Psi13_hb2_m * invGamma_b2_m * e2 / n
	C12_p     = h_opt[1,1]^2 * D12_hb2_p * (-0.1)
	C12_m     = h_opt[1,2]^2 * D12_hb2_m * (-0.1)
	
	// Compute C13_1 ===========================================================
	
	C131_p = h_opt[1,1]^3 * D12_hb2_p * B31_const
	C131_m = h_opt[1,2]^3 * D12_hb2_m * B31_const
	
	// Compute C13_2 ===========================================================
	
	if (h_opt[1,1] <= b3[1,1]) {
		Kvec_b3_p13 = select(Kvec_b3_p, Kind_h_p)
		Wb3_p13     = diag(Kvec_b3_p13)
		X3b3_p13    = select(X3b3_p, Kind_h_p) 
		Psi13_hb3_p = s2_p * X3h_p2' * Wh_p * Wb3_p13 * X3b3_p13 / n
	}
	else {
		Kvec_h_p2   = select(Kvec_h_p, Kind_b3_p)
		Wh_p13      = diag(Kvec_h_p2);
		X3h_p13     = select(X3h_p[.,(1,2)], Kind_b3_p)
		Psi13_hb3_p = s2_p * X3h_p13' * Wh_p13 * Wb3_p * X3b3_p2 / n
	}
	if (h_opt[1,2] <= b3[1,2]) {
		Kvec_b3_m13 = select(Kvec_b3_m, Kind_h_m)
		Wb3_m13     = diag(Kvec_b3_m13);    
		X3b3_m13    = select(X3b3_m, Kind_h_m)
		Psi13_hb3_m = s2_m * X3h_m2' * Wh_m * Wb3_m13 * X3b3_m13 / n
	}
	else {
		Kvec_h_m2   = select(Kvec_h_m, Kind_b3_m)
		Wh_m13      = diag(Kvec_h_m2);
		X3h_m13     = select(X3h_m[.,(1,2)], Kind_b3_m)
		Psi13_hb3_m = s2_m * X3h_m13' * Wh_m13 * Wb3_m * X3b3_m2 / n
	}
	
	D13_hb3_p = b3[1,1]^(-3) * e0' * invGamma_h_p * Psi13_hb3_p * invGamma_b3_p * e3 / n
	D13_hb3_m = b3[1,2]^(-3) * e0' * invGamma_h_m * Psi13_hb3_m * invGamma_b3_m * e3 / n
	C132_p    = h_opt[1,1]^3 * D13_hb3_p * (-0.1)
	C132_m    = h_opt[1,2]^3 * D13_hb3_m * (-0.1)
	
	// Compute C23_1 ===========================================================
	
	C231_p = h_opt[1,1]^5 * V23_b_p * B012_p * (-0.1) * B31_const
	C231_m = h_opt[1,2]^5 * V23_b_m * B012_m * (-0.1) * B31_const
	
	// Compute C23_2 ===========================================================
	
	if (b2[1,1] <= b3[1,1]) {
		Kvec_b3_p23 = select(Kvec_b3_p, Kind_b2_p)
		Wb3_p23     = diag(Kvec_b3_p23)
		X3b3_p23    = select(X3b3_p, Kind_b2_p)
		Psi33_b23_p = s2_p * X3b2_p2' * Wb2_p * Wb3_p23 * X3b3_p23 / n
	}
	else {
		Kvec_b2_p23 = select(Kvec_h_p,Kind_b3_p)
		Wb2_p23     = diag(Kvec_b2_p23)
		X3b2_p23    = select(X3b2_p, Kind_b3_p)
		Psi33_b23_p = s2_p * X3b2_p23' * Wb2_p23 * Wb3_p * X3b3_p2 / n
	}
	if (b2[1,2] <= b3[1,2]) {
		Kvec_b3_m23 = select(Kvec_b3_m, Kind_b2_m)
		Wb3_m23     = diag(Kvec_b3_m23)
		X3b3_m23    = select(X3b3_m, Kind_b2_m)
		Psi33_b23_m = s2_m * X3b2_m2' * Wb2_m * Wb3_m23 * X3b3_m23 / n
	}
	else {
		Kvec_b2_m23 = select(Kvec_b2_m, Kind_b3_m)
		Wb2_m23     = diag(Kvec_b2_m23);
		X3b2_m23    = select(X3b2_m, Kind_b3_m)
		Psi33_b23_m = s2_m * X3b2_m23' * Wb2_m23 * Wb3_m * X3b3_m2 / n
	}
	
	D23_hb23_p = b2[1,1]^(-2) * b3[1,1]^(-3) * e2' * invGamma_b2_p * Psi33_b23_p * invGamma_b3_p * e3 / n
	D23_hb23_m = b2[1,2]^(-2) * b3[1,2]^(-3) * e2' * invGamma_b2_m * Psi33_b23_m * invGamma_b3_m * e3 / n
	C232_p = h_opt[1,1]^5 * D23_hb23_p * (0.01)
	C232_m = h_opt[1,2]^5 * D23_hb23_m * (0.01)
	
	// Compute C33 =============================================================
	
	C33_p = h_opt[1,1]^6 * D23_hb23_p * (-0.1) * B31_const
	C33_m = h_opt[1,2]^6 * D23_hb23_m * (-0.1) * B31_const
	
	// Compute the robust variance =============================================
	
	Vrbc_a = V1 + V2 + V31 + V32
	Vrbc_b = - C12_p - C12_m - C131_p + C131_m - C132_p + C132_m
	Vrbc_c = C231_p - C231_m + C232_p - C232_m + C33_p + C33_m
	Vrbc   = Vrbc_a + 2*(Vrbc_b + Vrbc_c)
	se_r   = sqrt(Vrbc)
	
	bias1_p     = h_opt[1,1]^2 * m2_p * (-0.1) / 2
	bias1_m     = h_opt[1,2]^2 * m2_m * (-0.1) / 2
	bias21_p    = h_opt[1,1]^3 * m2_p * B31_const / 2
	bias21_m    = h_opt[1,2]^3 * m2_m * B31_const / 2
	bias22_p    = h_opt[1,1]^3 * m3_p * (-0.1) / 6
	bias22_m    = h_opt[1,2]^3 * m3_m * (-0.1) / 6
	b11_vec_opt = bias1_p + bias21_p + bias22_p
	b12_vec_opt = bias1_m - bias21_m - bias22_m
	bias_opt    = b11_vec_opt - b12_vec_opt
	
	// Compute the undersmoothed CI ============================================
	
	k = 6.5
	hus      = n^(-1/k) * h_mmse
	hus_ind_p = (data_mat_p[.,3]      :< hus[1,1])
	hus_ind_m = (abs(data_mat_p[.,3]) :< hus[1,2])
	n_lt_h_p = sum(hus_ind_p)
	n_lt_h_m = sum(hus_ind_m)
	
	while ((n_lt_h_p <= 2) | (n_lt_h_m <= 2)) {
		k = k + 1/2
		hus       = n^(-1/k) * h_mmse
		hus_ind_p = (data_mat_p[.,3]      :< hus[1,1])
		hus_ind_m = (abs(data_mat_p[.,3]) :< hus[1,2])
		n_lt_h_p  = sum(hus_ind_p)
		n_lt_h_m  = sum(hus_ind_m)
	}
	
	// Compute tau_us and V1_us ================================================
	
	e0 = (1 \ 0)
	hus_vec_p  = (1, (1/hus[1,1]), (1/hus[1,1])^2, (1/hus[1,1])^3)
	hus_vec_m  = (1, (1/hus[1,2]), (1/hus[1,2])^2, (1/hus[1,2])^3)
	Hus_p3     = diag(hus_vec_p)
	Hus_m3     = diag(hus_vec_m)
	X3hus_p    = data_mat_p[.,(2..5)] * Hus_p3
	X3hus_m    = data_mat_m[.,(2..5)] * Hus_m3
	Kind_hus_p = (abs(data_mat_p[.,3]) :< hus[1,1])
	Kind_hus_m = (abs(data_mat_m[.,3]) :< hus[1,2])
	Kvec_hus_p = (data_mat_p[.,2] - abs(data_mat_p[.,3]) / hus[1,1]) :* Kind_hus_p
	Kvec_hus_m = (data_mat_m[.,2] - abs(data_mat_m[.,3]) / hus[1,2]) :* Kind_hus_m
	yhus_p     = select(data_mat_p[.,1], Kind_hus_p)
	yhus_m     = select(data_mat_m[.,1], Kind_hus_m)
	X3hus_p2   = select(X3hus_p[.,(1,2)], Kind_hus_p)
	X3hus_m2   = select(X3hus_m[.,(1,2)], Kind_hus_m)
	Kvec_hus_p2 = select(Kvec_hus_p, Kind_hus_p)
	Kvec_hus_m2 = select(Kvec_hus_m, Kind_hus_m)
	Whus_p      = diag(Kvec_hus_p2)
	Whus_m      = diag(Kvec_hus_m2)
	Gamma_hus_p = X3hus_p2' * Whus_p * X3hus_p2 / n
	Gamma_hus_m = X3hus_m2' * Whus_m * X3hus_m2 / n
	invGamma_hus_p = invsym(Gamma_hus_p)
	invGamma_hus_m = invsym(Gamma_hus_m)
	XWY_hus_p   = X3hus_p2' * Whus_p * yhus_p / n
	XWY_hus_m   = X3hus_m2' * Whus_m * yhus_m / n
	delta_us_p_opt = e0' * invGamma_hus_p * XWY_hus_p
	delta_us_m_opt = e0' * invGamma_hus_m * XWY_hus_m
	tau_us      = delta_us_p_opt - delta_us_m_opt
	beta_us_p   = invGamma_hus_p * XWY_hus_p
	beta_us_m   = invGamma_hus_m * XWY_hus_m
	YT_us_p_hat = X3hus_p2 * beta_us_p
	YT_us_m_hat = X3hus_m2 * beta_us_m
	s2_us_p     = (yhus_p - YT_us_p_hat)' * Whus_p * (yhus_p - YT_us_p_hat) / trace(Whus_p - Whus_p * X3hus_p2 * invsym(X3hus_p2' * Whus_p * X3hus_p2) * X3hus_p2' * Whus_p)
	s2_us_m     = (yhus_m - YT_us_m_hat)' * Whus_m * (yhus_m - YT_us_m_hat) / trace(Whus_m - Whus_m * X3hus_m2 * invsym(X3hus_m2' * Whus_m * X3hus_m2) * X3hus_m2' * Whus_m)
	
	Psi_hus_p   = s2_us_p * X3hus_p2' * Whus_p * Whus_p * X3hus_p2 / n
	Psi_hus_m   = s2_us_m * X3hus_m2' * Whus_m * Whus_m * X3hus_m2 / n
	Vhus_p      = (e0' * invGamma_hus_p * Psi_hus_p * invGamma_hus_p * e0) / n
	Vhus_m      = (e0' * invGamma_hus_m * Psi_hus_m * invGamma_hus_m * e0) / n
	V1_us       = Vhus_p + Vhus_m
	se_us       = sqrt(V1_us)
	
	// Construct bias-corrected t-value ========================================
	
	tval_r  = (tau - bias_opt) / se_r
	pval_r  = 2 * (1 - normal(abs(tval_r)))
	
	ci_r_m  = tau    - bias_opt - 1.96*se_r
	ci_r_p  = tau    - bias_opt + 1.96*se_r
	ci_c_m  = tau_c  - bias_opt - 1.96*se_c
	ci_c_p  = tau_c  - bias_opt + 1.96*se_c
	ci_nv_m = tau               - 1.96*se_c
	ci_nv_p = tau               + 1.96*se_c
	ci_us_m = tau_us            - 1.96*se_us
	ci_us_p = tau_us            + 1.96*se_us
	
	st_numscalar("se_c",    se_c)
	st_numscalar("tau_r",   tau)
	st_numscalar("se_r",    se_r)
	st_numscalar("t_r",     tval_r)
	st_numscalar("p_r",     pval_r)
	st_numscalar("ci_r_m",  ci_r_m)
	st_numscalar("ci_r_p",  ci_r_p)
	st_numscalar("ci_c_m",  ci_c_m)
	st_numscalar("ci_c_p",  ci_c_p)
	st_numscalar("ci_nv_m", ci_nv_m)
	st_numscalar("ci_nv_p", ci_nv_p)
	st_numscalar("ci_us_m", ci_us_m)
	st_numscalar("ci_us_p", ci_us_p)
	st_numscalar("hus_m",   hus[1,2])
	st_numscalar("hus_p",   hus[1,1])
	st_numscalar("Nus_m",   n_lt_h_m)
	st_numscalar("Nus_p",   n_lt_h_p)
	st_numscalar("Nopt_m",  num_yh_m)
	st_numscalar("Nopt_p",  num_yh_p)
	
	st_numscalar("bias_opt", bias_opt)
}
end

mata
real scalar f_k(x, h) {
	n = rows(x)
	V = J(n,1,1)
	A = x / h
	E = (3/4) * sum((abs(A):<1) :* (V - A:*A))
	r = E/n/h
	return(r)
}
end

mata
real scalar f1_pilot(x, h) {
	n = rows(x)
	A = -x / h
	B = A:^2
	V = J(n,1,1)
	E = -3/4*sum(A:*(V-B):*(abs(A):<1))/(1/5)
	R = E/n/h^2
	return(R)
}
end

